MORLAB - Model Order Reduction Laboratory
=========================================

The MORLAB, Model Order Reduction LABoratory, toolbox is a collection of 
MATLAB routines for model order reduction of dynamical systems based on 
the solution of matrix equations.
The implementation is made using spectral projection methods, e.g.,
methods based on the matrix sign function and the matrix disk function.


## Getting Started
------------------

### Dependencies

Matlab R2012b and above, Octave 4.0.0 and above.

Generally, the implemented functions run fine in plain Matlab/Octave.
However, to use the ss functionality of the toolbox, the Control
System Toolbox (Matlab) or Control Package (Octave) is needed.

Also, some parts of the demo scripts use functions from the Control
System Toolbox (in Octave: Control Package 3.0.0 and above) and the Robust
Control Toolbox. In case of using the demo scripts without the necessary
toolboxes, the corresponding commands won't be called.


### Installation

Change to the MORLAB main folder, containing this file, and run the
`ml_addpath` function. This will add this folder together with all
subfolders to your Matlab/Octave search path. If you do not want
to do this every time, you need to save the resulting search path.


### Using MORLAB

The toolbox includes several methods for model reduction of linear
time-invariant continuous-time systems in standard or descriptor form

      Ex'(t) = Ax(t) + Bu(t),
        y(t) = Cx(t) + Du(t).

For the case E = I, there is a set of adjusted routines given. For the
complete toolbox it is assumed that the matrix pencil s*E - A has no purely
imaginary eigenvalues if the implemented equation solvers are used. Note
that also eigenvalues close to the imaginary axis might cause problems.

For each implemented model order reduction method there is a demo script
to get into the use of MORLAB. In the following, the different techniques
are listed up with the corresponding functions and demo scripts:

1. Modal Truncation (MT)
    * `ml_mt` for standard systems with the demo script
      `morlab_demo_mt.m`
    * `ml_mt_dss` for descriptor systems with the demo script
      `morlab_demo_mt_dss.m`

2. Balanced Truncation (BT)
    * `ml_bt` for standard systems with the demo script
      `morlab_demo_bt.m`
    * `ml_bt_dss` for descriptor systems with the demo script
      `morlab_demo_bt_dss.m`

3. Bounded-Real Balanced Truncation (BRBT)
    * `ml_brbt` for standard systems with the demo script
      `morlab_demo_brbt.m`
    * `ml_brbt_dss` for descriptor systems with the demo script
      `morlab_demo_brbt_dss.m`

4. Positive-Real Balanced Truncation (PRBT)
    * `ml_prbt` for standard systems with the demo script
      `morlab_demo_prbt.m`
    * `ml_prbt_dss` for descriptor systems with the demo script
      `morlab_demo_prbt_dss.m`

5. Balanced Stochastic Truncation (BST)
    * `ml_bst` for standard systems with the demo script
      `morlab_demo_bst.m`
    * `ml_bst_dss` for descriptor systems with the demo script
      `morlab_demo_bst_dss.m`

6. Linear-Qadratic-Gaussian Balanced Truncation (LQGBT)
    * `ml_lqgbt` for standard systems with the demo script
      `morlab_demo_lqgbt.m`
    * `ml_lqgbt_dss` for descriptor systems with the demo script
      `morlab_demo_lqgbt_dss.m`

7. H-infinity Balanced Truncation (HINFBT)
    * `ml_hinfbt` for standard systems with the demo script
      `morlab_demo_hinfbt.m`
    * `ml_hinfbt_dss` for descriptor systems with the demo script
      `morlab_demo_hinfbt_dss.m`

8. Hankel-Norm Approximation (HNA)
    * `ml_hna` for standard systems with the demo script
      `morlab_demo_hna.m`
    * `ml_hna_dss` for descriptor systems with the demo script
      `morlab_demo_hna_dss.m`

Furthermore, the toolbox contains several methods in the following folder
structure:
  * **checks**: checks of used variable types
  * **demos**: demo scripts and data for model reduction functions
  * **eqn_solvers**: matrix equation solvers
  * **mor**: model reduction routines
  * **subroutines**: subroutines/further functions for dynamical systems


### MORLAB Style Guide

In the codes of the toolbox a special naming and formatting style was
applied, which might be useful to know for further developments and use of
the implemented routines.

For the naming of routines one should know:
  * All function names are lower case and concatenated by underscores.
  * All functions start with an `ml_`.
  * All scripts and data files start with a `morlab_`.
  * The functions corresponding to descriptor systems end with `_dss`.
  * The names of equation solver routines are made up the following way
    `ml_<name of equation>_<method>[_<fac>]`, where the underlying method
    is abbreviated, e.g., sgn for sign function iterations and nwt for
    newton iterations. The last term `[_<fac>]` is optional and indicates
    that the solution is computed in a factorized form.

For variables one should know:
  * For the entries of structs it holds:
      - Simple data types (scalars, matrices, strings, etc.) are written in
        camelCase beginning with an upper case letter.
      - Structs for inputs are all lower case and usually end with `opts`.
      - Structs for outputs are camelCase beginning with an `info` in lower
        case and the corresponding computation routine in all upper case.
  * For other variables it holds:
      - Matrices are written camelCase beginning with an upper letter.
      - All other variables are lower case.


## Authors
----------

See [CONTRIBUTORS.md](CONTRIBUTORS.md) for details about the authors of
the toolbox.


## License
----------

Copyright (C) 2006-2017 Peter Benner, Steffen W. R. Werner

The software is licensed under AGPLv3 or later.
See [COPYING](COPYING) for a copy of the license.


## Contact
----------

URL: http://www.mpi-magdeburg.mpg.de/projects/morlab

email: benner@mpi-magdeburg.mpg.de, werner@mpi-magdeburg.mpg.de

orcids: 0000-0003-3362-4103, 0000-0003-1667-4862


## Citation
-----------

See [CITATION.md](CITATION.md) for details about citing the software.
